#!/usr/bin/perl
use lib '/home/martin/ethereal/mgmt';

#################################################################################
# Created       : Martin Foster
# Modified      : 04-Nov-2005
#################################################################################
#
# Dice - Script part of Ethereal Realms, designed to send die rolls
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Comm;						# Communication handler
use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Option handling
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handler

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

#################################################################################
# Data Members
#################################################################################
my $comm;							# Communications panel handle
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $tmpl;							# Template handle

my %realm;							# Realm hash

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);


	# Parameter handling
	$param    = new Ethereal::Param($database, $cgi);
	$param->GetParam();	

	# Fetch hash
	$database->GetHashRealm(\%realm);

	# Login and Comm handle
	$tmpl     = new Ethereal::Template(\%realm);
	$login    = new Ethereal::Login($database, $cgi, $param); 
	$comm     = new Ethereal::Comm($database, $cgi, $tmpl);
	
	
	# Authenthication
	if ($login->GetVerificationRealm())
	{
		# Prevent bans from use
		$login->CheckLegality(\%realm);

		# Determine and assign
		my ($tsize, $tcolor, $tface) = $database->DataGetFont($param->{'CHAR'}, $param->{'USER'});

		# Set handles
		$param->{'COLOR'} = $tcolor;
		$param->{'FACE'}  = $tface;
		$param->{'SIZE'}  = 3;


		# Only do the following if posting is needed
		if (defined($param->{'ROLL'}))
		{
			RollHandler($param, \%realm);
			$comm->DataWrite($param);
		}

		# Display panel
		DiceDisplay($database, $cgi, $param, \%realm);
	}


#################################################################################
# Sub-Routines
#################################################################################

####################
# Dice Display
#
# Gives the users options for the roll of dice.  This means the selection of the
# type of die as well as the number of specific rolls.

sub DiceDisplay
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Cgi handle
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Chat options

	my $option;						# Option handler
	my $access;						# Access control

	my $blow;						# Below
	my $btch;						# Botches
	my $diff;						# Difficulty
	my $damg;						# Damage modifier
	my $type;						# Selection type
	my $freq;						# Frequency

	my $panel;						# Dice roller panel

	my $dlowr = $param->{'DLOWR'};				# Drop lowest
	my $dhigh = $param->{'DHIGH'};				# Drop highest

	my $dblow = $param->{'DBLOW'};				# Lower then calculations
	my $dbtch = $param->{'DBTCH'};				# Damage modifier
	my $ddamg = $param->{'DDAMG'};				# Damage modifier
	my $ddiff = $param->{'DDIFF'};				# Dice difficulty
	my $dfreq = $param->{'DFREQ'};				# Dice frequency
	my $dtype = $param->{'DTYPE'};				# Dice type

	my @char;						# Character selection
	my @types;						# Dice types


	####################
	# Program area

	# Create instance
	$option   = new Ethereal::Option($cgi);

	# Find types
	$option->Split(\@types, $realm->{'OptDiceTypes'});

	# Assign values
	$blow = (defined($dbtch)) ? $dblow : 0;
	$btch = (defined($dbtch)) ? $dbtch : 0;
	$damg = (defined($ddamg)) ? $ddamg : 0;
	$diff = (defined($ddiff)) ? $ddiff : 0;
	$freq = (defined($dfreq)) ? $dfreq : 1;
	$type = (defined($dtype)) ? $dtype : $types[0];

	$dlowr = (defined($dlowr)) ? $dlowr : 0;
	$dhigh = (defined($dhigh)) ? $dhigh : 0;	


	# Populate puppet list
	($access) = $database->DataGetRealmAccess($param->{'ROOM'});

	# Build widget
	if ($access ne 'restricted')
	{
		# Pull list of all applicable handles
		$database->GetListPuppetRealm(\@char, $param->{'USER'}, $param->{'ROOM'});
	}
	else
	{
		# Pull list of canon handles
		$database->GetListCanonPuppets(\@char, $param->{'USER'}, $param->{'ROOM'});
	}

	
        # Retreive dice roller panel
        $panel = $database->FetchRealmDice($param->{'ROOM'});


        # Widgets
	# Character selection
	my $wcharslct = $cgi->popup_menu('CHAR', \@char, $param->{'CHAR'});

        # Assigning values
        my $wdiceroll = $cgi->submit($realm->{'TxtRoll'});

	# Basics
        my $wdicefreq = $cgi->textfield('DFREQ', $freq, 3, 2);
        my $wdicetype = $cgi->popup_menu('DTYPE',  \@types,  $type); 

	# Success
	my $wdicediff = $cgi->textfield('DDIFF', $diff, 3, 2);
	my $wdicebtch = $cgi->textfield('DBTCH', $btch, 3, 2);

	# Damage modifer
	my $wdicedamg = $cgi->textfield('DDAMG', $damg, 3, 2);
	my $wdiceblow = $cgi->textfield('DBLOW', $blow, 3, 2);
	
	# Drops
	my $wdicelowr = $cgi->textfield('DLOWR', $dlowr, 3, 2);
	my $wdicehigh = $cgi->textfield('DHIGH', $dhigh, 3, 2);

	# Private Message
	my $wdivepvte = (defined($param->{'DPVTE'}))
	 ? $cgi->checkbox(-name=>'DPVTE', -label=>'', -checked=>'on')
	 : $cgi->checkbox(-name=>'DPVTE', -label=>'');

	# Individual Count
	my $wdiveindv = (defined($param->{'DINDV'}))
	 ? $cgi->checkbox(-name=>'DINDV', -label=>'', -checked=>'on')
	 : $cgi->checkbox(-name=>'DINDV', -label=>'');

	# Cleanup
	$param->Cleanup('DPVTE','DINDV');


	# Form values
	my $wformstrt = $cgi->start_form();
	my $wformhidn = $param->EmbedNormal($param->Flat(), ROLL=>'True');
	my $wformends = $cgi->end_form();


	# Character select
	$panel =~ s/WCHARSLCT/$wcharslct/gs;

	# Basics
	$panel =~ s/WDICEFREQ/$wdicefreq/gs;
	$panel =~ s/WDICETYPE/$wdicetype/gs;

	# Success
	$panel =~ s/WDICEDIFF/$wdicediff/gs;
	$panel =~ s/WDICEBTCH/$wdicebtch/gs;

	# Damage modifier
	$panel =~ s/WDICEBLOW/$wdiceblow/gs;
	$panel =~ s/WDICEDAMG/$wdicedamg/gs;

	# Drops
	$panel =~ s/WDICELOWR/$wdicelowr/gs;
	$panel =~ s/WDICEHIGH/$wdicehigh/gs;

	# Checkboxes
	$panel =~ s/WDICEPVTE/$wdivepvte/gs;
	$panel =~ s/WDICEINDV/$wdiveindv/gs;

	# Submit
	$panel =~ s/WDICEROLL/$wdiceroll/gs;

	# Form
	$panel =~ s/WFORMSTRT/$wformstrt$wformhidn/gs;
	$panel =~ s/WFORMENDS/$wformends/gs;



	# Display HTML
	# Document
	print $panel, "\n";
}


####################
# Roll Handler
#
# Will perform a roll of the dice as specified by the user himself.   This is done
# using a pseudo random seed.

sub RollHandler
{
	####################
	# Data members
	my $param = shift;					# Parameter handle
	my $realm = shift;					# Chat options

	my $indv  = 0;						# Individual addition
	my $mes;						# Simple message
	my $result;						# Singular result
	my $results;						# Formatted results
	my $total;						# Active total

	my $btc;						# Botches
	my $dif;						# Difficulty
	my $max;						# Max damage

	my $botches;						# Number of botches
	my $count;						# Counter
	my $damage;						# Damage modifier string
	my $success;						# Number of sucesses
	my $rand;						# Random result

	my @results;						# Array of results


	####################
	# Program area

	# Adjust seed
	srand();

	# Ensure defined
	$param->{'DBLOW'} = (defined($param->{'DBLOW'})) ? $param->{'DBLOW'} : 0;
	$param->{'DBTCH'} = (defined($param->{'DBTCH'})) ? $param->{'DBTCH'} : 1;
	$param->{'DDAMG'} = (defined($param->{'DDAMG'})) ? $param->{'DDAMG'} : 0;
	$param->{'DDIFF'} = (defined($param->{'DDIFF'})) ? $param->{'DDIFF'} : 0;
	$param->{'DFREQ'} = (defined($param->{'DFREQ'})) ? $param->{'DFREQ'} : 1;
	$param->{'DLOWR'} = (defined($param->{'DLOWR'})) ? $param->{'DLOWR'} : 0;
	$param->{'DHIGH'} = (defined($param->{'DHIGH'})) ? $param->{'DHIGH'} : 0;

	# Make sure numbers are received
	$param->{'DBLOW'} = ($param->{'DBLOW'} =~ /^\d+$/) ? $param->{'DBLOW'} : 0;
	$param->{'DBTCH'} = ($param->{'DBTCH'} =~ /^\d+$/) ? $param->{'DBTCH'} : 1;
	$param->{'DDAMG'} = ($param->{'DDAMG'} =~ /^(-|\d)+$/) ? $param->{'DDAMG'} : 0;
	$param->{'DDIFF'} = ($param->{'DDIFF'} =~ /^\d+$/) ? $param->{'DDIFF'} : 0;
	$param->{'DFREQ'} = ($param->{'DFREQ'} =~ /^\d+$/) ? $param->{'DFREQ'} : 1;
	$param->{'DLOWR'} = ($param->{'DLOWR'} =~ /^\d+$/) ? $param->{'DLOWR'} : 0;
	$param->{'DHIGH'} = ($param->{'DHIGH'} =~ /^\d+$/) ? $param->{'DHIGH'} : 0;

	# Number cap
	$param->{'DFREQ'} = ($param->{'DFREQ'} < 250) ? $param->{'DFREQ'} : 250;
	$param->{'DDAMG'} = ($param->{'DDAMG'} < 50)  ? $param->{'DDAMG'} :  50;

	# Difficulty cannot surpass type
	$param->{'DDIFF'} = ($param->{'DDIFF'} > ($param->{'DTYPE'}+$param->{'DDAMG'})) ? 0 : $param->{'DDIFF'};

	# Botches cannot be higher then diff
	$param->{'DBTCH'} = ($param->{'DBTCH'} < $param->{'DDIFF'}) ? $param->{'DBTCH'} : 0;

	

	# Drops cannot surpass dice
	if (($param->{'DLOWR'} + $param->{'DHIGH'}) > ($param->{'DFREQ'} - 1))
	{
		# Reset them to zero
		$param->{'DLOWR'} = 0;
		$param->{'DHIGH'} = 0;
	}

	# Max score listing
	$max = ($param->{'DBLOW'} > 0) ? " " . $realm->{'TxtDiceMax'} . ":" . $param->{'DBLOW'} : '';

	# Difficultly listing
	$dif = ($param->{'DDIFF'} > 0) ? " " . $realm->{'TxtDiceDiff'} . ":" . $param->{'DDIFF'} : '';

	# Botch listing
	$btc = ($param->{'DBTCH'} > 0) ? " " . $realm->{'TxtDiceBtch'} . ":" . $param->{'DBTCH'} : '';


	# Damage modifer handling
	if ($param->{'DDAMG'} != 0)
	{
		# Individual market
		$indv = (defined($param->{'DINDV'})) ? $realm->{'TxtDiceIndividual'} : '';


		# Presentation
		# Positive modifer
		if ($param->{'DDAMG'} > 0)
		{ $damage = "+$param->{DDAMG}$indv"; }

		# Negative modifier
		elsif ($param->{'DDAMG'} < 0)
		{ $damage = "$param->{DDAMG}$indv"; }

		# Simple spacing
		else
		{ $damage = ''; }


		# Individual addition calc
		if (defined($param->{'DINDV'}))
		{ $indv  = $param->{'DDAMG'}; }

		# Total
		else
		{ $total = $param->{'DDAMG'}; }
	}


	# Comments
	if ((defined($param->{'DPOST'}))
	 && (length($param->{'DPOST'}) > 3))
	{
		# Append comments
		$results = "<B>" . $realm->{'TxtDiceAction'} . "</B> " . $param->{'DPOST'} . "<BR>\n";
	}
	else
	{
		# Empty string
		$results = '';
	} 

	# Contruct begin results
	$rand    = int((rand() * $param->{'DTYPE'}) + 1 + $indv);
	$results = $results . "<B>" . $realm->{TxtDiceRoll} . " </B>(<B>$param->{DFREQ}d$param->{DTYPE}$damage$dif$btc$max</B>) " . $rand;
	$total  += $rand;

	# Append to array
	push(@results, $rand);

	# Rolls for each roll
	for ($count = 1; $count < $param->{'DFREQ'}; $count++)
	{
		# Generate number
		$rand = int((rand() * $param->{'DTYPE'}) + 1 + $indv);


		# Continue contruct
		$results = "$results, $rand";

		# Work on total
		$total += $rand;

		# Append to array
		push(@results, $rand);
	} 

	# Sort array
	@results = sort Numeric @results;

	
	# Initial Results
	$results = "$results ($total)<BR>";

	# Drops
	# Lowest
	if ($param->{'DLOWR'} > 0)
	{
		# Reset counter
		$count = 1;


		# Pull initial result
		$result = shift(@results);
		$total -= $result;

		# Format
		$results = $results . "<BR><B>$realm->{TxtDiceDrop} " . $param->{'DLOWR'} . " $realm->{TxtDiceLow}</B>: " . $result;


		# Loop until out of elements or reached count
		while (($result = shift(@results)) and ($count < $param->{'DLOWR'}))
		{
			# Drop from result
			$total -= $result;

			# Massage string
			$results = $results . ", $result";

			# Increment counter
			$count++;
		}

		# Final format
		$results = $results . " (" . $total . ")";
	}

	# Highest
	if ($param->{'DHIGH'} > 0)
	{
		# Reset counter
		$count = 1;


		# Pull initial result
		$result = pop(@results);
		$total -= $result;

		# Format
		$results = $results . "<BR><B>$realm->{TxtDiceDrop} " . $param->{'DHIGH'} . " $realm->{TxtDiceHigh}</B>: " . $result;


		# Loop until out of elements or reached count
		while (($result = pop(@results)) and ($count < $param->{'DHIGH'}))
		{
			# Drop from result
			$total -= $result;

			# Massage string
			$results = $results . ", $result";

			# Increment counter
			$count++;
		}

		# Final format
		$results = $results . " (" . $total . ")";
	}


	# Lower then
	if ($param->{'DBLOW'} > 0)
	{
		# Successfully lower
		if ($total > $param->{'DBLOW'})
		{ $results = $results . "<BR><BR><B>" . $realm->{'TxtDiceFail'} . "</B>"; }

		# Failed because higher
		else
		{ $results = $results . "<BR><BR><B>" . $realm->{'TxtDiceSuccess'} . "</B>"; }
	}


	# Botches and sucesses
	if ($param->{'DDIFF'} > 0)
	{
		# Additional padding
		$results = (($param->{'DHIGH'} > 0) or ($param->{'DLOWR'} > 0)) ? "$results<BR>" : $results;

		# Determine amounts
		while ($result = shift(@results))
		{
			# Success and botches
			$success += ($result >= $param->{'DDIFF'}) ? 1 : 0;
			$botches += ($result < ($param->{'DBTCH'}) + 1) ? 1 : 0;
		}


		# Botches
		if ($param->{'DBTCH'} > 0)
		{
			# Botch
			if ($botches > $success)
			{ $mes = "<B>" . $realm->{'TxtDiceBotch'} . "</B> ($botches)" ; }

			# Success
			elsif ($botches < $success)
			{ $mes = "<B>" . $realm->{'TxtDiceSuccessRoll'} . "</B> ($success)"; }

			# Failed
			else
			{ $mes = "<B>" . $realm->{'TxtDiceFailRoll'} . "</B>"; }
		}

		# Simple success
		else
		{
			# Successful message
			$mes = "<B>" . $realm->{'TxtDiceSuccess'} . "</B> ($success)";
		}

		# Display message
		$results = $results . "<BR>$mes";
	}


	# Assign contruct as post
	$param->{'POST'}   = $results;
	$param->{'TARGET'} = 'dice roll';
}


####################
# Numeric
#
# Numeric comparison in order to be able to sort by numbers.   

sub Numeric
{ $a <=> $b; } 
